#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "common.h"
#include "macros.h"
#include "failures.h"
#include "vm.h"
#include "bo.h"

#define	MEM_SIZE	2048

extern char *__progname;
static unsigned char ios[MEM_SIZE];

static void print_usage()
{
	printf("Usage: %s test_file.v\n", __progname);
}

int main(int argc, char **argv)
{
	struct vm *machine;
	const char *msg[] = { "ok", "failed" };
	char *file_name;
	ret_t ret;
	int res;

	if (argc < 2) {
		print_usage();
		exit(0);
	}

	file_name = argv[1];

	machine = vm_create();
	if (machine == NULL)
		xerror(1, "can't create machine");

	ret = vm_io_area_set(machine, ios, sizeof(ios), ios, sizeof(ios), BO_NATIVE);
	if (ret != ret_ok)
		xerror(1, "can't set inputs/outputs areas");

	ret = vm_load_file(machine, file_name, "test", 1);
	if (ret != ret_ok)
		xerror(1, "can't load file");

	ret = vm_prepare(machine);
	if (ret != ret_ok)
		xerror(1, "machine is not prepared");

	memset(machine->stack, 0xdeadbeef, machine->stack_size >> 2);
	vm_stack_shrink(machine, 8);

	vm_run(machine);

	vm_stack_grow(machine, 4);

	res = vm_stack_get_int(machine);

	if (res != 0 && res != 1)
		xerror(1, "return unknown value %d", res);

	printf("%s: %s -- %s\n", __progname, file_name, msg[res]);

	ret = vm_unload_file(machine);
	if (ret != ret_ok)
		xerror(1, "can't unload file");

	ret = vm_io_area_unset(machine);
	if (ret != ret_ok)
		xerror(1, "can't unset inputs/outputs areas");

	ret = vm_destroy(&machine);
	if (ret != ret_ok)
		xerror(1 ,"can't destroy machine");

	return 0;
}

